"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

ImageView subclass:#ImageEditView
	instanceVariableNames:'magnification colors magnifiedImage colorPanel'
	classVariableNames:''
	poolDictionaries:''
	category:'Views-Misc'
!

!ImageEditView  class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This View will eventually be able to edit bitmap images.
    For now, it is not.
"
! !

!ImageEditView methodsFor:'accessing'!

image:anImage
    magnification := 1@1.
    magnifiedImage := nil.
    super image:anImage
! !

!ImageEditView methodsFor:'drawing'!

redrawX:x y:y width:w height:h
    |area ih iw dotW dotH minX maxX minY maxY color last lastY runW x0 oldClip|

    image isNil ifTrue:[^ self].

"/    area := Rectangle left:x top:y width:w height:h.      
"/    oldClip := self clippingRectangleOrNil.
"/    self clippingRectangle:area.

    (magnification = (1@1)) ifTrue:[
        super redrawX:x y:y width:w height:h.
"/        self clippingRectangle:oldClip.
        ^ self
    ].
        
"/    self clear.

    magnifiedImage notNil ifTrue:[
        magnifiedImage := magnifiedImage on:device.

        magnifiedImage depth == 1 ifTrue:[
            self paint:(magnifiedImage colorFromValue:1)
                    on:(magnifiedImage colorFromValue:0).
        ] ifFalse:[
            self paint:Black on:White.
        ].

        self function:#copy.
        self displayOpaqueForm:magnifiedImage x:margin y:margin.
"/        self clippingRectangle:oldClip.
        ^ self
    ].

"/    self clear.
    ih := image height.
    iw := image width.
    dotW := magnification x.
    dotH := magnification y.

    minX := x // dotW.
    minY := y // dotW.
    maxX := (x + w) // dotW + 1.
    maxX > iw ifTrue:[
        maxX := iw
    ].
    maxY := (y + h) // dotH + 1.
    maxY > ih ifTrue:[
        maxY := ih
    ].

    lastY := -1.

    x0 := minX.
    runW := 0.

    image colorsFromX:minX y:minY toX:maxX-1 y:maxY-1 do:[:x :y :color |
        y ~~ lastY ifTrue:[
            runW ~~ 0 ifTrue:[
                self fillRectangleX:(x0 * dotW + margin)
                                  y:(lastY * dotH + margin)
                              width:runW height:dotH.
                runW := 0.
            ].
            x0 := x.
            lastY := y.
        ].

        color ~~ last ifTrue:[
            runW ~~ 0 ifTrue:[
                self fillRectangleX:(x0 * dotW + margin)
                                  y:(y * dotH + margin)
                              width:runW height:dotH.
                runW := 0.
            ].

            "/ keep colors - otherwise they might get collected & changed
            colors notNil ifTrue:[colors add:color].
            self paint:color.
            last := color.
            runW := 0.
            x0 := x.
        ].
        runW := runW + dotW
    ].

    runW ~~ 0 ifTrue:[
        self fillRectangleX:(x0 * dotW + margin)
                          y:(lastY * dotH + margin)
                      width:runW height:dotH.
        runW := 0.
    ].

"/    self clippingRectangle:oldClip

    "Modified: 11.7.1996 / 21:13:53 / cg"
! !

!ImageEditView methodsFor:'event handling'!

buttonMotion:state x:x y:y
    self showColorAtX:x y:y.
!

buttonMultiPress:button x:x y:y
    button == 1 ifTrue:[
        |clr|
        clr := self getColorAtX:x y:y.
        clr inspect.
    ].
    super buttonMultiPress:button x:x y:y

    "Created: 8.5.1996 / 00:18:06 / stefan"
!

buttonPress:button x:x y:y
    button == 1 ifTrue:[
	self showColorAtX:x y:y.
	^ self
    ].
    super buttonPress:button x:x y:y
!

getColorAtX:x y:y
    |pi|

    pi := (x @ y) - margin.
    pi := pi // magnification.
    ((0@0 corner:image extent) containsPoint:pi) ifTrue:[
        ^ image at:pi.
    ].
    ^ nil

    "Created: 8.5.1996 / 00:15:55 / stefan"
!

showColorAtX:x y:y
    |clr|

    clr := self getColorAtX:x y:y.
    clr notNil ifTrue:[
        colorPanel notNil ifTrue:[
            colorPanel setColor:clr.
        ] ifFalse:[
            Transcript showCR:clr displayString
        ]
    ]

    "Modified: 8.5.1996 / 00:20:45 / stefan"
! !

!ImageEditView methodsFor:'image processing'!

blurr
    self performImageOperation:#blurr withArguments:nil
!

flipHorizontal
    self performImageOperation:#flipHorizontal withArguments:nil
!

flipVertical
    self performImageOperation:#flipVertical withArguments:nil
!

performImageOperation:operation withArguments:args
    |oldMag|

    windowGroup withCursor:Cursor wait do:[
        oldMag := magnification.
        magnifiedImage := nil.
        magnification := 1@1.
        image perform:operation withArguments:args.
        (oldMag isNil or:[oldMag = magnification]) ifTrue:[
            self invalidate
        ] ifFalse:[
            self magnification:oldMag.
        ]
    ]

    "Modified: 11.7.1996 / 21:03:10 / cg"
! !

!ImageEditView methodsFor:'initialization'!

initialize
    super initialize.
    magnification := 1@1.
    colors := nil
!

initializeMiddleButtonMenu
    |m|

    m := (PopUpMenu
	       labels:(resources array:
			  #('save as ...'
			    '-'
			    'magnification'
"/                            'colors'
			    'effects'
			   ))
	    selectors:#(saveAs
			nil
			changeMagnification
"/                        showColors
			effects
		       )
	     receiver:self
		  for:self).
    self middleButtonMenu:m.

    m subMenuAt:#effects put:(
	PopUpMenu labels:(resources array:#(
			    'flip - vertical'
			    'flip - horizontal'
"
			    '-'
			    'blurr'
"
			   ))
	       selectors:#(
			    flipVertical
			    flipHorizontal
"
			    nil
			    blurr
"
			   )
		receiver:self
		     for:self

    ).

    "
     ImageEditView openOn:'bitmaps/SBrowser.xbm'
     ImageEditView openOn:'bitmaps/garfield.gif'
    "
! !

!ImageEditView methodsFor:'menu actions'!

changeMagnification
    |b|

    b := EnterBox new.
    b title:'magnification (magX @ magY)'.
    b okText:'apply'.
    b abortText:'abort'.
    b action:[:string | self magnification:(Object readFromString:string)].
    b initialText:(magnification printString).
    b showAtPointer
!

magnification:aMagnificationPoint
    |nPixel savedImage|

    magnification ~= aMagnificationPoint ifTrue:[
        "show wait cursor; although magnification is fast, dithering may take a while"
        self cursor:Cursor wait.

        (device visualType == #PseudoColor) ifTrue:[
            "release kept colors"
            colors := IdentitySet new.
        ].

        magnification := aMagnificationPoint asPoint.

        "use a magnified image, if its size wont be too big"
        magnifiedImage := nil.

        "avoid slow scroll"
        savedImage := image.
        image := nil.
        self scrollToTopLeft.
        image := savedImage.

        magnification ~= (1@1) ifTrue:[
            nPixel := image width * image height * magnification x * magnification y.
            nPixel <= (2000 * 2000) ifTrue:[
                Transcript showCR:'magnifying ..'; endEntry.
                magnifiedImage := image magnifiedBy:magnification
            ].
        ].

        self contentsChanged.
        self invalidate.
        self cursor:Cursor normal
    ]

    "Modified: 5.7.1996 / 17:44:29 / stefan"
    "Modified: 11.7.1996 / 21:03:06 / cg"
!

saveAs
    "save contents into a file 
     - ask user for filename using a fileSelectionBox."

    |fileName|

    fileName := Dialog
		    requestFileName:'save image in:'
		    default:''
		    ok:'save'
		    abort:'abort'
		    pattern:'*.tiff'.

    fileName notNil ifTrue:[
	image saveOn:fileName
    ].
!

showColors
    colorPanel isNil ifTrue:[
	colorPanel := ColorPanel new
    ].
    colorPanel shown ifFalse:[
	colorPanel realize
    ]
! !

!ImageEditView methodsFor:'queries'!

heightOfContents
    "return the images height"

    image isNil ifTrue:[^ 0].
    ^ (image height * magnification y) rounded
!

widthOfContents
    "return the images width"

    image isNil ifTrue:[^ 0].
    ^ (image width * magnification x) rounded
! !

!ImageEditView methodsFor:'release'!

destroy
    colors := nil.
    magnifiedImage := nil.
    colorPanel notNil ifTrue:[
	colorPanel destroy.
	colorPanel := nil.
    ].
    super destroy.
! !

!ImageEditView  class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/ImageEditView.st,v 1.26 1996-07-11 19:15:00 cg Exp $'
! !
